package main

import (
	"log"
	"math"
	"net/http"
	"os"
	"strconv"
	"strings"

	"github.com/gin-gonic/gin"
	"github.com/youpy/go-wav"
)

const sampleRate = 44100.0 // サンプルレートを指定
const bpm = 120            // テンポを指定
func main() {
	// サーバーの設定
	router := gin.Default()
	// オルゴールの画面(HTMLファイル)を返す --- (*1)
	router.StaticFS("/static", http.Dir("static"))
	router.GET("/", func(ctx *gin.Context) {
		ctx.Redirect(302, "/static/index.html")
	})
	// WAVファイル生成API --- (*2)
	router.GET("/make", func(ctx *gin.Context) {
		notesStr := ctx.Query("notes")
		notes := strings.Split(notesStr, ",")
		file, _ := os.Create("static/box.wav")
		defer file.Close()
		len4 := int(sampleRate * 60 / bpm)
		sampleSize := uint32(len(notes) * len4)
		writer := wav.NewWriter(file, sampleSize, 1, uint32(sampleRate), 16)
		for i := 0; i < len(notes); i++ {
			// 各音符を書き込む
			no, _ := strconv.Atoi(notes[i])
			addNote(writer, no, len4)
		}
		ctx.String(200, "ok")
	})
	// サーバーを起動 --- (*3)
	err := router.Run("127.0.0.1:8888")
	if err != nil {
		log.Fatal("サーバー起動に失敗", err)
	}
}

// 音符を書き込む --- (*4)
func addNote(writer *wav.Writer, note int, length int) {
	// 音階から周波数を計算
	tone := 440.0 * math.Pow(2.0, float64(note-69)/12.0)
	// サイン波を書き込む
	samples := make([]wav.Sample, length)
	for i := 0; i < length; i++ {
		v := math.Sin((float64(i) / float64(sampleRate)) * tone * 2.0 * math.Pi)
		v *= 0.3 // 音量を下げる
		// プチッと切れるノイズを軽減
		decLen := length / 8 // 音符の最後の音量を下げる
		if i > length-decLen {
			v *= float64((length - i)) / float64(decLen)
		}
		samples[i].Values[0] = int(v * 0x7FFF)
	}
	writer.WriteSamples(samples)
}
